import json

from Acquisition import aq_inner
from Acquisition import aq_parent
from Acquisition import aq_base
from ftw.file.utils import is_image
from plone import api
from plone.outputfilters.browser.resolveuid import uuidFor
from Products.CMFCore.interfaces._content import IFolderish
from Products.CMFCore.utils import getToolByName
from Products.TinyMCE.adapters.Upload import Upload
from zExceptions import BadRequest
from zope.event import notify
from zope.i18n import translate
from Products.Archetypes.event import ObjectEditedEvent
from zope.publisher.browser import BrowserView

from ftw.file import fileMessageFactory as _

import pkg_resources
try:
    pkg_resources.get_distribution('plone.dexterity')
except pkg_resources.DistributionNotFound:
    HAS_DEXTERITY = False
else:
    HAS_DEXTERITY = True
    from plone.dexterity.interfaces import IDexterityContent


class FileUpload(BrowserView):
    """View for handling drag-and-drop file replace"""

    def __call__(self):
        self.file = self.request.get('file')
        if not self.file:
            raise BadRequest('No content provided.')

        self.filename = self.file.filename
        self.context.update(file=self.file, originFilename=self.filename)

        portal = api.portal.get()
        repository_tool = getToolByName(portal, 'portal_repository')

        if repository_tool.isVersionable(self.context):
            # TODO: This creates another entry in the history resulting
            # in two consecutive history entries.
            repository_tool.save(
                self.context,
                comment=translate(_('File replaced with Drag & Drop.'),
                                  context=self.request)
            )

        notify(ObjectEditedEvent(self.context))
        return json.dumps({'success': True})


class TinyMCEFileUpload(Upload):

    def upload(self):
        """Adds uploaded file.

        Required params: uploadfile, uploadtitle, uploaddescription
        """
        context = aq_inner(self.context)
        self.request = context.REQUEST
        if not IFolderish.providedBy(context):
            context = aq_parent(context)

        request = context.REQUEST
        utility = getToolByName(context, 'portal_tinymce')

        id_ = request['uploadfile'].filename
        content_type = request['uploadfile'].headers["Content-Type"]
        # check if container is ready to store images
        if self.is_temporary(context):
            return self.errorMessage(
                translate(_('Please save the object first'
                            ' to enable image upload.'),
                          context=self.request))

        # check mime type to make sure an image is uploaded
        if not is_image(content_type):
            return self.errorMessage(
                translate(_('Only image upload allowed.'),
                          context=self.request))

        # Permission checks based on code by Danny Bloemendaal

        # 1) check if the current user has permissions to add stuff
        if not context.portal_membership.checkPermission(
            'Add portal content', context):
            return self.errorMessage(
                "You do not have permission to upload files in this folder")

        # 2) check image types uploadable in folder.
        #    priority is to content_type_registry image type
        allowed_types = [t.id for t in context.getAllowedTypes()]
        tiny_image_types = utility.imageobjects.split('\n')
        uploadable_types = []
        for typename in tiny_image_types:
            if typename in allowed_types:
                uploadable_types.append(typename)

        # Get an unused filename without path
        id_ = self.cleanupFilename(id_)

        for metatype in uploadable_types:
            try:
                newid = context.invokeFactory(type_name=metatype, id=id_)
                if newid is None or newid == '':
                    newid = id_
                break
            except ValueError:
                continue
            except BadRequest:
                return self.errorMessage(
                    translate(_("Bad filename, please rename."),
                              context=self.request))
        else:
            return self.errorMessage(translate(
                _("Not allowed to upload a file of this type to this folder"),
                context=self.request))

        obj = getattr(context, newid, None)

        # Set title + description.
        # Attempt to use Archetypes mutator if there is one, in case it uses
        # a custom storage
        title = request['uploadtitle']
        description = request['uploaddescription']

        if description:
            try:
                obj.setDescription(description)
            except AttributeError:
                obj.description = description

        if HAS_DEXTERITY and IDexterityContent.providedBy(obj):
            if not self.setDexterityImage(obj):
                return self.errorMessage(translate(
                    _("The content-type '%s' has no image-field!" % metatype),
                    context=self.request))
        else:
            # set primary field
            pf = obj.getPrimaryField()
            pf.set(obj, request['uploadfile'])

        if not obj:
            return self.errorMessage("Could not upload the file")

        if title and title is not '':
            obj.setTitle(title)
        else:
            obj.setTitle(obj.getFilename())

        obj.reindexObject()
        folder = obj.aq_parent.absolute_url()

        if utility.link_using_uids:
            path = "resolveuid/%s" % (uuidFor(obj))
        else:
            path = obj.absolute_url()
        return self.okMessage(path, folder)

    def is_temporary(self, obj, checkId=True):
        """Checks, whether an object is a temporary object (means it's in the
        `portal_factory`) or has no acquisition chain set up.
        Source: http://svn.plone.org/svn/collective/collective.indexing
        /trunk/collective/indexing/subscribers.py
        """
        parent = aq_parent(aq_inner(obj))
        if parent is None:
            return True
        if checkId and getattr(obj, 'getId', None):
            if getattr(aq_base(parent), obj.getId(), None) is None:
                return True
        isTemporary = getattr(obj, 'isTemporary', None)
        if isTemporary is not None:
            try:
                if obj.isTemporary():
                    return True
            except TypeError:
                # `isTemporary` on the `FactoryTool` expects 2 args
                return True
        return False



import json
from StringIO import StringIO
from Products.CMFCore.utils import getToolByName

from ftw.builder import Builder
from ftw.builder import create
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase
from zExceptions import BadRequest

from ftw.file.testing import FTW_FILE_INTEGRATION_TESTING


class TestAjaxUpload(TestCase):

    layer = FTW_FILE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.file_ = create(
            Builder('file')
            .titled('Document')
            .attach_file_containing('My PDF', name='test.pdf')
        )

        self.new_file = StringIO('Raw file data')
        self.new_file.filename = 'new_file.txt'
        self.new_file.content_type = 'text/plain'

    def test_replace_file(self):
        self.portal.REQUEST.set('file', self.new_file)

        ajax_upload_view = self.file_.restrictedTraverse('ajax-upload')
        response = json.loads(ajax_upload_view())

        self.assertEqual(response['success'], True)

    def test_no_file(self):
        with self.assertRaises(BadRequest):
            self.file_.restrictedTraverse('ajax-upload')()

    def test_new_version(self):
        portal = api.portal.get()
        repository_tool = getToolByName(portal, 'portal_repository')
        repository_tool.setVersionableContentTypes('File')

        self.portal.REQUEST.set('file', self.new_file)
        self.file_.restrictedTraverse('ajax-upload')()

        history = repository_tool.getHistory(self.file_)
        self.assertEqual(1, len(history))



from OFS.Image import File
from PIL.Image import ANTIALIAS
from ftw.file.events.events import FileDownloadedEvent
from ftw.file.imaging import ImagingMixin
from ftw.file.interfaces import IFtwFileField
from plone.app.blob import field
from plone.app.blob.download import handleIfModifiedSince, handleRequestRange
from plone.registry.interfaces import IRegistry
from urllib import quote
from webdav.common import rfc1123_date
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.event import notify
from zope.interface import implements


class FileField(field.FileField, ImagingMixin):
    implements(IFtwFileField)

    _properties = field.FileField._properties.copy()
    _properties.update({
        'pil_quality': 88,
        'pil_resize_algo': ANTIALIAS,
        'content_class': File,

    })

    def index_html(self, instance, REQUEST=None, RESPONSE=None,
                   charset='utf-8', disposition='inline'):
        """Kicks download.
        Writes data including file name and content type to RESPONSE
        """

        if REQUEST is None:
            REQUEST = instance.REQUEST

        if RESPONSE is None:
            RESPONSE = REQUEST.RESPONSE

        RESPONSE.setHeader('Last-Modified', rfc1123_date(instance._p_mtime))
        RESPONSE.setHeader('Content-Type', self.getContentType(instance))
        RESPONSE.setHeader('Accept-Ranges', 'bytes')

        if handleIfModifiedSince(instance, REQUEST, RESPONSE):
            return ''

        length = self.get_size(instance)
        RESPONSE.setHeader('Content-Length', length)

        filename = self.getFilename(instance)
        if filename is not None:
            if isinstance(filename, unicode):
                filename = filename.encode(charset, errors="ignore")

            # Create a user agent specific disposition header
            # IE needs an url quoted filename
            # Other browsers need an unquoted filename
            user_agent = REQUEST.get('HTTP_USER_AGENT', '')
            if 'MSIE' in user_agent:
                header_value = '%s; filename=%s' % (disposition,
                                                    quote(filename))
            else:
                header_value = '%s; filename="%s"' % (disposition, filename)
            RESPONSE.setHeader("Content-disposition", header_value)

        request_range = handleRequestRange(instance, length, REQUEST, RESPONSE)

        # Notify file downloads, but do not notify range requests
        if not ('start' in request_range and request_range['start'] > 0):
            portal_state = getMultiAdapter((instance, instance.REQUEST),
                                           name='plone_portal_state')
            if not portal_state.anonymous():
                registry = getUtility(IRegistry)
                user_ids = registry['ftw.file.filesettings.user_ids']
                if portal_state.member().id \
                        and not portal_state.member().id in user_ids:
                    notify(FileDownloadedEvent(instance, filename))

        return self.get(instance).getIterator(**request_range)



import transaction

from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import setRoles
from plone.testing.z2 import Browser
from unittest2 import TestCase
from pyquery import PyQuery


class TestAdvancedEdit(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

        self.portal = self.layer['portal']

        transaction.commit()

    def login(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD,))

    def test_no_advanced_edit(self):
        setRoles(self.portal, TEST_USER_ID, ['Contributor', 'Editor'])
        transaction.commit()
        self.login()
        self.browser.open(self.portal.absolute_url() + '/createObject?type_name=File')
        html = PyQuery(self.browser.contents)
        fieldsets = html('fieldset')
        self.assertEqual(len(fieldsets), 0, "We found Fieldsets. We didn't expect any'")

    def test_advanced_edit(self):
        form_tabs = ["Default",
                     "Categorization",
                     "Dates",
                     "Creators",
                     "Settings",
                     "Ownership"
                     ]
        setRoles(self.portal, TEST_USER_ID, ['Contributor', 'Editor', 'Reviewer'])
        transaction.commit()
        self.login()
        self.browser.open(self.portal.absolute_url() + '/createObject?type_name=File')
        html = PyQuery(self.browser.contents)
        fieldsets = html('fieldset')
        self.assertEqual(len(fieldsets), 5, "We expected 5 Fieldsets got %s" % len(fieldsets))
        legends = fieldsets('legend')
        for legend in legends:
            self.assertIn(legend.text, form_tabs)
